/* e_elem1.cc
 *$Header: /al/acs/src/RCS/e_elem1.cc,v 8.1 95/04/23 18:20:01 al Exp $
 * Base class for elements of a circuit
 * This file contains functions to load the matrix from a circuit element
 */
#include "ecah.h"
#include "m_matrix.h"
#include "error.h"
#include "e_elemnt.h"
#include "u_option.h"
#include "u_status.h"
#include "s__.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
//	void	ELEMENT::trload_loss();
//	void	ELEMENT::trunload_loss();
//	void	ELEMENT::acload_loss();
//	void	ELEMENT::trload_passive();
//	void	ELEMENT::trunload_passive();
//	void	ELEMENT::acload_passive();
//	void	ELEMENT::trload_active();
//	void	ELEMENT::trunload_active();
//	void	ELEMENT::acload_active();
//	void	ELEMENT::trload_source();
//	void	ELEMENT::trunload_source();
//	void	ELEMENT::acload_source();
/*--------------------------------------------------------------------------*/
extern       struct nodestuff ns;
extern BSMATRIXdouble aa, lu;
extern BSMATRIXcomplex acx;
/*--------------------------------------------------------------------------*/
void ELEMENT::trload_loss()
{
  if (!SIM::inc_mode  &&  loss != 0.){
    if (n[OUT2].m != 0){
      aa.d(n[OUT2].m,n[OUT2].m) += loss;
      if (n[OUT1].m != 0){
	aa.d(n[OUT1].m,n[OUT1].m) += loss;
	aa.m(n[OUT1].m,n[OUT2].m)  -= loss;
	aa.m(n[OUT2].m,n[OUT1].m)  -= loss;
      }
    }else if (n[OUT1].m != 0){
      aa.d(n[OUT1].m,n[OUT1].m) += loss;
    }
  }
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trunload_loss()
{
  assert(!"trunload_loss");
}
/*--------------------------------------------------------------------------*/
void ELEMENT::acload_loss()
{
  if (n[OUT2].m != 0){
    acx.d(n[OUT2].m,n[OUT2].m) += loss;
    if (n[OUT1].m != 0){
      acx.d(n[OUT1].m,n[OUT1].m) += loss;
      acx.m(n[OUT1].m,n[OUT2].m)  -= loss;
      acx.m(n[OUT2].m,n[OUT1].m)  -= loss;
    }
  }else if (n[OUT1].m != 0){
    acx.d(n[OUT1].m,n[OUT1].m) += loss;
  }
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trload_passive()
{
  double dc0;
  double df1;
  if (loaditer == STATUS::iter[iTOTAL])
    error(bDANGER, "%s: double load\n", printlabel());
  loaditer = STATUS::iter[iTOTAL];
  
  if (STATUS::iter[iSTEP] > 1  &&  !nodamp){
    dc0 = OPT::damp * (m0.c0 - m1.c0);
    m0.c0 = m1.c0 + dc0;
    df1 = OPT::damp * (m0.f1 - m1.f1);
    m0.f1 = m1.f1 + df1;
    if (!SIM::inc_mode){
      dc0 = m0.c0;
      df1 = m0.f1;
    }
  }else{
    if (SIM::inc_mode){
      dc0 = m0.c0 - m1.c0;
      df1 = m0.f1 - m1.f1;
    }else{
      dc0 = m0.c0;
      df1 = m0.f1;
    }
  }
  if (df1 != 0.){
    if (n[OUT2].m != 0){
      setneedslu(OUT2);
      aa.d(n[OUT2].m,n[OUT2].m) += df1;
      if (n[OUT1].m != 0){
	setneedslu(OUT1);
	aa.d(n[OUT1].m,n[OUT1].m) += df1;
	aa.m(n[OUT1].m,n[OUT2].m)  -= df1;
	aa.m(n[OUT2].m,n[OUT1].m)  -= df1;
      }
    }else if (n[OUT1].m != 0){
      setneedslu(OUT1);
      aa.d(n[OUT1].m,n[OUT1].m) += df1;
    }
  }
  if (dc0 != 0.){
    if (n[OUT2].m != 0)
      ns.i[n[OUT2].m] += dc0;
    if (n[OUT1].m != 0)
      ns.i[n[OUT1].m] -= dc0;
  }
  m1 = m0;
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trunload_passive()
{
  m0.c0 = m0.f1 = 0.;
  if (SIM::inc_mode)
    SIM::inc_mode = BAD;
  trload_passive();
}
/*--------------------------------------------------------------------------*/
void ELEMENT::acload_passive()
{
  if (n[OUT2].m != 0){
    acx.d(n[OUT2].m,n[OUT2].m) += acg;
    if (n[OUT1].m != 0){
      acx.d(n[OUT1].m,n[OUT1].m) += acg;
      acx.m(n[OUT1].m,n[OUT2].m)  -= acg;
      acx.m(n[OUT2].m,n[OUT1].m)  -= acg;
    }
  }else if (n[OUT1].m != 0){
    acx.d(n[OUT1].m,n[OUT1].m) += acg;
  }
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trload_active()
{
  double dc0;
  double df1;
  if (loaditer == STATUS::iter[iTOTAL])
    error(bDANGER, "%s: double load\n", printlabel());
  loaditer = STATUS::iter[iTOTAL];

  if (STATUS::iter[iSTEP] > 1  &&  !nodamp){
    dc0 = OPT::damp * (m0.c0 - m1.c0);
    m0.c0 = m1.c0 + dc0;
    df1 = OPT::damp * (m0.f1 - m1.f1);
    m0.f1 = m1.f1 + df1;
    if (!SIM::inc_mode){
      dc0 = m0.c0;
      df1 = m0.f1;
    }
  }else{
    if (SIM::inc_mode){
      dc0 = m0.c0 - m1.c0;
      df1 = m0.f1 - m1.f1;
    }else{
      dc0 = m0.c0;
      df1 = m0.f1;
    }
  }
  if (df1 != 0.){
    setneedslu(OUT1);
    setneedslu(OUT2);
    setneedslu(IN1);
    setneedslu(IN2);
    aa.s(n[OUT1].m,n[IN1].m) += df1;
    aa.s(n[OUT2].m,n[IN2].m) += df1;
    aa.s(n[OUT1].m,n[IN2].m) -= df1;
    aa.s(n[OUT2].m,n[IN1].m) -= df1;
  }
  if (dc0 != 0.){
    if (n[OUT2].m != 0)
      ns.i[n[OUT2].m] += dc0;
    if (n[OUT1].m != 0)
      ns.i[n[OUT1].m] -= dc0;
  }
  m1 = m0;
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trunload_active()
{
  m0.c0 = m0.f1 = 0.;
  if (SIM::inc_mode)
    SIM::inc_mode = BAD;
  trload_active();
}
/*--------------------------------------------------------------------------*/
void ELEMENT::acload_active()
{
  acx.s(n[OUT1].m,n[IN1].m) += acg;
  acx.s(n[OUT2].m,n[IN2].m) += acg;
  acx.s(n[OUT1].m,n[IN2].m) -= acg;
  acx.s(n[OUT2].m,n[IN1].m) -= acg;
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trload_source()
{
  double dc0;
  if (loaditer == STATUS::iter[iTOTAL])
    error(bDANGER, "%s: double load\n", printlabel());
  loaditer = STATUS::iter[iTOTAL];
  
  if (STATUS::iter[iSTEP] > 1  &&  !nodamp){
    dc0 = OPT::damp * (m0.c0 - m1.c0);
    m0.c0 = m1.c0 + dc0;
    if (!SIM::inc_mode){
      dc0 = m0.c0;
    }
  }else{
    if (SIM::inc_mode){
      dc0 = m0.c0 - m1.c0;
    }else{
      dc0 = m0.c0;
    }
  }
  if (dc0 != 0.){
    if (n[OUT2].m != 0)
      ns.i[n[OUT2].m] += dc0;
    if (n[OUT1].m != 0)
      ns.i[n[OUT1].m] -= dc0;
  }
  m1 = m0;
}
/*--------------------------------------------------------------------------*/
void ELEMENT::trunload_source()
{
  m0.c0 = m0.f1 = 0.;
  if (SIM::inc_mode)
    SIM::inc_mode = BAD;
  trload_source();
}
/*--------------------------------------------------------------------------*/
void ELEMENT::acload_source()
{
  if (n[OUT2].m != 0){
    ns.ac[n[OUT2].m] += acg;
  }
  if (n[OUT1].m != 0){
    ns.ac[n[OUT1].m] -= acg;
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
