/* main.c
 $Header: /al/acs/src/RCS/main.cc,v 8.6 95/05/07 20:10:05 al Exp $
 * top level module
 * it all starts here
 */
#include "ecah.h"
#include "ap.h"
#include "error.h"
#include "io.h"
#include "mode.h"
#include "u_option.h"
#include "patchlev.h"
#include "u_timer.h"
#include "c_comand.h"
#include "s_dc.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
	int	main(int,const char*[]);
static	void    sign_on(void);
static	void    read_startup_files(void);
static	void    process_cmd_line(int,const char*[]);
static	void	finish(void);
	void	cmdproc(const char*);
/*--------------------------------------------------------------------------*/
extern const char e_int[];
extern FILE *stream[];	    /* reverse of fileno() */
extern jmp_buf envp;
extern int cmdcount;
extern int run_mode;
/*--------------------------------------------------------------------------*/
int main(int argc, const char *argv[])
{
 initialize_io();
 sign_on();
 if (setjmp(envp)){		
    finish();			/* error clean up (from longjmp())	*/
 }else{
    read_startup_files();
    setup_traps();
    process_cmd_line(argc,argv);
 }  
 for (;;){
    char cmdbuf[BUFLEN];
    cmdcount++;
    (void)getcmd("-->", cmdbuf, BUFLEN);
    run_mode = rEXECUTE;
    cmdproc(cmdbuf);
 }
 /*NOTREACHED*/
 return 0;
}
/*--------------------------------------------------------------------------*/
static void sign_on(void)
{
 mprintf(IO::mstdout,"ACS (Al's Circuit Simulator) 0.%u\n",PATCHLEVEL);
 mprintf(IO::mstdout,"Never trust any version less than 1.0\n");
 mprintf(IO::mstdout,"Copyright 1994, Albert Davis\n");
 mprintf(IO::mstdout,"ACS comes with ABSOLUTELY NO WARRANTY\n");
 mprintf(IO::mstdout,"This is free software, and you are welcome\n");
 mprintf(IO::mstdout,"to redistribute it under certain conditions.\n");
 mprintf(IO::mstdout,"See the file \"COPYING\" for details\n");
}
/*--------------------------------------------------------------------------*/
static void read_startup_files(void)
{
 char *name;
 if (!!(name = findfile(SYSTEMSTARTFILE, SYSTEMSTARTPATH, R_OK))){
    char cmdbuf[BUFLEN];
    sprintf(cmdbuf, "get %s", name);
    cmdproc(cmdbuf);
 }
 if (!!(name = findfile(USERSTARTFILE, USERSTARTPATH, R_OK))){
    char cmdbuf[BUFLEN];
    sprintf(cmdbuf, "get %s", name);
    cmdproc(cmdbuf);
 }
 cmdproc("clear");
}
/*--------------------------------------------------------------------------*/
static void process_cmd_line(int argc, const char *argv[])
{
 if (argc > 1){
    char cmdbuf[BUFLEN];
    sprintf(cmdbuf, "< %s", argv[1]);
    cmdproc(cmdbuf);
 }
}
/*--------------------------------------------------------------------------*/
/* finish: clean up after a command
 * deallocates space, closes plot windows, resets i/o redirection, etc.
 * This is done separately for exception handling.
 * If a command aborts, clean-up is still done, leaving a consistent state.
 */
static void finish(void)
{
 DCOP::finish();
 plclose();
 IO::suppresserrors = NO;
 outreset();
 dealloc(NO);
}
/*--------------------------------------------------------------------------*/
/* cmdproc: process a command
 * parse, and act on, a command string
 */
void cmdproc(const char *cmd)
{
 CS cs(cmd);
 static TIMER timecheck;
 int didsomething = YES;

 error(bTRACE, "%s\n", cmd);

 timecheck.check();
 timecheck.zstart();
 if (cs.argparse(ONEPASS,
	"Ac",	    aFFUNCTION,	 CMD::ac,
	"ALArm",    aFFUNCTION,	 CMD::alarm,
	"ALTer",    aFFUNCTION,	 CMD::alter,
	"Build",    aFFUNCTION,  CMD::build,
	"CHDir",    aFFUNCTION,  CMD::chdir,
	"CDir",	    aFFUNCTION,  CMD::chdir,
	"CLEAR",    aFFUNCTION,  CMD::clear,
	"CRTSET",   aFFUNCTION,  CMD::crtset,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"DC",	    aFFUNCTION,	 CMD::dc,
	"DELete",   aFFUNCTION,  CMD::del,
	"DIsto",    aFFUNCTION,	 CMD::disto,
	"Edit",	    aFFUNCTION,  CMD::edit,
	"END",	    aFFUNCTION,	 CMD::end,
	"ENDS",	    aFFUNCTION,	 CMD::ends,
	"EXIt",	    aFFUNCTION,  CMD::quit,
	"FANout",   aFFUNCTION,  CMD::fanout,
	"FAult",    aFFUNCTION,	 CMD::fault,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"FOurier",  aFFUNCTION,	 CMD::fourier,
	"Generator",aFFUNCTION,  CMD::generator,
	"GET",	    aFFUNCTION,  CMD::get,
	"Help",	    aFFUNCTION,  CMD::help,
	"IC",	    aFFUNCTION,	 CMD::ic,
	"INsert",   aFFUNCTION,  CMD::insert,
	"List",	    aFFUNCTION,  CMD::list,
	"LOg",	    aFFUNCTION,	 CMD::logger,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"MArk",	    aFFUNCTION,	 CMD::mark,
	"MErge",    aFFUNCTION,  CMD::merge,
	"MODEl",    aFFUNCTION,	 CMD::model,
	"Modify",   aFFUNCTION,  CMD::modify,
	"NODeset",  aFFUNCTION,	 CMD::nodeset,
	"NOIse",    aFFUNCTION,	 CMD::noise,
	"OP",	    aFFUNCTION,	 CMD::op,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"OPTions",  aFFUNCTION,  CMD::options,
	"PAuse",    aFFUNCTION,	 CMD::pause,
	"PLot",	    aFFUNCTION,  CMD::plot,
	"PRint",    aFFUNCTION,  CMD::print,
	"PRobe",    aFFUNCTION,  CMD::print,
	"Quit",	    aFFUNCTION,  CMD::quit,
	"Restore",  aFFUNCTION,  CMD::restore,
	"SAve",	    aFFUNCTION,  CMD::save,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"SENs",	    aFFUNCTION,	 CMD::sens,
	"SEt",	    aFFUNCTION,  CMD::options,
	"SPectrum", aFFUNCTION,	 CMD::fourier,
	"STatus",   aFFUNCTION,  CMD::status,
	"SUbckt",   aFFUNCTION,	 CMD::subckt,
	"SWeep",    aFFUNCTION,  CMD::sweep,
	"TEmp",	    aFFUNCTION,	 CMD::temp,
	"TF",	    aFFUNCTION,	 CMD::tf,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	"TItle",    aFFUNCTION,  CMD::title,
	"TRansient",aFFUNCTION,	 CMD::tr,
	"UNFault",  aFFUNCTION,	 CMD::unfault,
	"UNMark",   aFFUNCTION,	 CMD::unmark,
	"Width",    aFFUNCTION,	 CMD::options,
	"!",	    aFFUNCTION,  CMD::system,
	"<",	    aFFUNCTION,	 CMD::run,
	""))
    ;	/* split to hide sun lint bug. */
 else if (cs.argparse(ONEPASS,
	">",	    aFFUNCTION,	 CMD::file,
	""))
    ;
 else{    /* comment or error */
    CMD::comment(cs);
    didsomething = NO;
 }
 if (OPT::acct  &&  didsomething){
    timecheck.check();
    mprintf(IO::mstdout,"time=%8.2f\n", timecheck.elapsed());
 }
 finish();
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
