/* s_ac_set.cc
 *$Header: /al/acs/src/RCS/s_ac_set.cc,v 8.1 95/04/23 18:20:30 al Exp $
 * ac analysis setup
 */
#include "ecah.h"
#include "ap.h"
#include "error.h"
#include "io.h"
#include "u_option.h"
#include "s_ac.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
//	void	AC::setup(CS&);
//	void	AC::by(CS&);
//	void	AC::decade(CS&);
//	void	AC::lin(CS&);
//	void	AC::octave(CS&);
//	void	AC::times(CS&);
/*--------------------------------------------------------------------------*/
static int needslinfix;	// flag: lin option needs patch later (spice compat)
static AC *This;	// fudge to work around argparse
/*--------------------------------------------------------------------------*/
void AC::setup(CS& cmd)
{
  This = this;
  IO::where |= IO::mstdout;
  temp = OPT::tempamb;
  alarmlist = PROBE_LISTS::alarm[mode];
  plotlist  = PROBE_LISTS::plot[mode];
  printlist = PROBE_LISTS::print[mode];
  storelist = PROBE_LISTS::store[mode];
  IO::ploton = IO::plotset  &&  plotlist.count() > 0;
  cold = cont = echo = NO;
  worstcase = wNONE;
  for (;;){
    if (cmd.argparse(REPEAT,
	"*",		aFFUNCTION, times,
    	"ACMAx",	aENUM,	    &worstcase,	wMAXAC,
	"ACMIn",	aENUM,	    &worstcase,	wMINAC,
	"Ambient",	aODOUBLE,   &temp,	OPT::tempamb,
	"By",		aFFUNCTION, by,
	"COLD",		aENUM,	    &cold,	YES,
	"CONTinue",	aENUM,	    &cont,	YES,
    	"DCMAx",	aENUM,	    &worstcase,	wMAXDC,
	"DCMIn",	aENUM,	    &worstcase,	wMINDC,
	"Decade",	aFFUNCTION, decade,
	"Echo",		aENUM,	    &echo,	YES,
	"LAg",		aENUM,	    &worstcase, wLAG,
	"LEad",		aENUM,	    &worstcase, wLEAD,
        "LIn",		aFFUNCTION, lin,
	"MAx",		aENUM,	    &worstcase, wMAXAC,
	"MIn",		aENUM,	    &worstcase, wMINAC,
	"NOPlot",	aENUM,	    &IO::ploton, NO,
	"Octave",	aFFUNCTION, octave,
	"PLot",		aENUM,	    &IO::ploton, YES,
	"Reftemp",	aODOUBLE,   &temp,	OPT::tnom,
	"SEnsitivity",	aENUM,	    &worstcase, wSENS,
	"Temperature",	aODOUBLE,   &temp,	-ABS_ZERO,
	"TImes",	aFFUNCTION, times,
	""))
	;	
    else if (cmd.is_float()){
      start = cmd.ctof();
      stop  = cmd.ctof();
      if (stop==0.)
	stop = start;
      if (cmd.is_float())
	by(cmd);
    }else if (outset(cmd,"","")){
      ;
    }else{
      cmd.check(bWARNING);
      break;
    }
  }
  initio(IO::where,IO::whence);
  if (worstcase==wRAND  ||  worstcase==wWORST)
    IO::ploton = NO;
  if (needslinfix){			    // LIN option is # of points.
    step=(stop-start)/(step-1.);	    // Must compute step after 
    needslinfix = NO;			    // reading start and stop,
  }					    // but step must be read first
  if (step==0.){			    // for Spice compatibility
    step = stop - start;
    linswp = YES;
  }
}
/*--------------------------------------------------------------------------*/
/*static*/ void AC::by(CS& cmd)
{
  This->step = cmd.ctof();
  needslinfix = NO;
  This->linswp = YES;
}
/*--------------------------------------------------------------------------*/
/*static*/ void AC::decade(CS& cmd)
{
  This->step = cmd.ctopf();
  if (This->step == 0.)
    This->step = 1.;
  This->step = pow(10., 1./This->step);
  needslinfix = NO;
  This->linswp = NO;
}
/*--------------------------------------------------------------------------*/
/*static*/ void AC::lin(CS& cmd)
{
  This->step = cmd.ctopf();		// need to fix step, later
  if (This->step == 0.)			// do it at the end of setup
    This->step = 1.;			// a kluge, but this is a patch
  needslinfix = YES;			// and I am too lazy to do it
  This->linswp = YES;			// right.
}
/*--------------------------------------------------------------------------*/
/*static*/ void AC::octave(CS& cmd)
{
  This->step = cmd.ctopf();
  if (This->step == 0.)
    This->step = 1.;
  This->step = pow(2.00000001, 1./This->step);
  needslinfix = NO;
  This->linswp = NO;
}
/*--------------------------------------------------------------------------*/
/*static*/ void AC::times(CS& cmd)
{
  This->step = cmd.ctopf();
  if (This->step == 0.   &&   This->start != 0.)
    This->step = This->stop / This->start;
  needslinfix = NO;
  This->linswp = NO;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
