/* solve.c
 $Header: /al/acs/src/RCS/solve.cc,v 8.1 95/04/23 18:20:39 al Exp $
 * LU decomposition and fwd-back substitution for real numbers
 * 2 modules:  lu() and solve()
 * lu() does LU decomposition using the Crout algorithm (modified)
 * solve() does forward and back substitution after calling lu() if necessary
 */
#include "ecah.h"
#include "error.h"
#include "e_node.h"
#include "u_option.h"
#include "u_status.h"
#include "s__.h"
#include "m_matrix.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
	void	solve(void);
static	void	lu_decomp(void);
static	void	lu_partial(void);
static	void	lu_full(void);
static	double	dotprod(int,int,int);
/*--------------------------------------------------------------------------*/
extern BSMATRIXdouble aa, lu;
extern       struct nodestuff ns;
extern struct nodestat *nstat;
/*--------------------------------------------------------------------------*/
void solve(void)
{
 int jj, ii;

 lu_decomp();
 STATUS::back.start();
 for (ii = 1;  ii <= lu.size();  ++ii){		/* forward substitution */
    ns.fw[ii] = ns.i[ii];
    for (jj = lu.lownode(ii);   jj < ii;   ++jj){
       ns.fw[ii] -= lu.l(ii,jj) * ns.fw[jj];
    }
    ns.fw[ii] /= lu.d(ii,ii);
 }
 if (OPT::foooo!=4){
    for (ii = lu.size();  ii >= 1;  --ii){
	  ns.v0[ii] = ns.fw[ii];
	  nstat[ii].aiter = STATUS::iter[iTOTAL];
    }
    for (jj = lu.size();  jj > 1;  --jj){	/* back substitution */
       for (ii = lu.lownode(jj);   ii < jj;   ++ii){
	   ns.v0[ii] -= lu.u(ii,jj) * ns.v0[jj];
       }
    }
 }else{
    for (ii = lu.size();  ii >= 1;  --ii){
       /*if (nstat[ii].mode != mGATE){*/
	  ns.v0[ii] = ns.fw[ii];
	  for (jj = ii+1;  jj <= lu.size();  ++jj){
	     ns.v0[ii] -= lu.s(ii,jj) * ns.v0[jj];
	  }
	  nstat[ii].aiter = STATUS::iter[iTOTAL];
	  /*nstat[ii].mode = mLU;*/
	  /*nstat[ii].lastchange = SIM::time0;*/
       /*}*/
    }
 }
 STATUS::back.stop();
 if (OPT::foooo == 1){
    for (ii=1; ii<=aa.size(); ++ii){
       for (jj=1; jj<=aa.size(); ++jj){
	  printf("%9.2e ", aa.s(ii,jj));
       }
       printf("%9.2e\n", ns.i[ii]);
    }
 }
}
/*--------------------------------------------------------------------------*/
static void lu_decomp(void)
{
 if (OPT::lubypass && SIM::inc_mode){
    lu_partial();
 }else{
    lu_full();
 }
}
/*--------------------------------------------------------------------------*/
static void lu_partial(void)
{
 int ii, jj, mm;
 int bn;
 int prop = 0;
 
 STATUS::lud.start();
 for (mm = 1;   mm <= aa.size();   ++mm){
    bn = aa.lownode(mm);
    if (nstat[mm].needslu  ||  bn <= prop){
       nstat[mm].needslu = NO;
       prop = mm;
       if (bn < mm){
	  prop = mm;
	  lu.u(bn,mm) = aa.u(bn,mm) / lu.d(bn,bn);
	  for (ii = bn+1;  ii<mm;  ii++){
	     lu.u(ii,mm) = (aa.u(ii,mm) - dotprod(ii,mm,ii)) / lu.d(ii,ii);
	  }
	  lu.l(mm,bn) = aa.l(mm,bn);
	  for (jj = bn+1;  jj<mm;  jj++){
	     lu.l(mm,jj) = aa.l(mm,jj) - dotprod(mm,jj,jj);
	  }
	  /* jj == mm */{
	     lu.d(mm,mm) = aa.d(mm,mm) - dotprod(mm,mm,mm);
	     if (OPT::gmin > lu.d(mm,mm)  &&  lu.d(mm,mm) > -OPT::gmin){
		error(bWARNING, "open circuit: internal node %u\n", mm);
		lu.d(mm,mm) = OPT::gmin;
	     }
	  }
       }else{    /* bn == mm */
	  lu.d(mm,mm) = aa.d(mm,mm);
	  if (OPT::gmin > lu.d(mm,mm)  &&  lu.d(mm,mm) > -OPT::gmin){
	     error(bWARNING, "open circuit: internal node %u\n", mm);
	     lu.d(mm,mm) = OPT::gmin;
	  }
       }
    }
 }
 STATUS::lud.stop();
}
/*--------------------------------------------------------------------------*/
static void lu_full(void)
{
 int ii, jj, mm;
 int bn;

 STATUS::lud.start();
 for (mm = 1;   mm <= aa.size();   ++mm){
    bn = aa.lownode(mm);
    if (bn < mm){
       lu.u(bn,mm) = aa.u(bn,mm) / lu.d(bn,bn);
       for (ii = bn+1;  ii<mm;  ii++){
	  lu.u(ii,mm) = (aa.u(ii,mm) - dotprod(ii,mm,ii)) / lu.d(ii,ii);
       }
       lu.l(mm,bn) = aa.l(mm,bn);
       for (jj = bn+1;  jj<mm;  jj++){
	  lu.l(mm,jj) = aa.l(mm,jj) - dotprod(mm,jj,jj);
       }
       /* jj == mm */{
	  lu.d(mm,mm) = aa.d(mm,mm) - dotprod(mm,mm,mm);
	  if (OPT::gmin > lu.d(mm,mm)  &&  lu.d(mm,mm) > -OPT::gmin){
	     error(bWARNING, "open circuit: internal node %u\n", mm);
	     lu.d(mm,mm) = OPT::gmin;
	  }
       }
    }else{    /* bn == mm */
       lu.d(mm,mm) = aa.d(mm,mm);
       if (OPT::gmin > lu.d(mm,mm)  &&  lu.d(mm,mm) > -OPT::gmin){
	  error(bWARNING, "open circuit: internal node %u\n", mm);
	  lu.d(mm,mm) = OPT::gmin;
       }
    }
 }
 STATUS::lud.stop();
}
/*--------------------------------------------------------------------------*/
static double dotprod(int ii, int jj, int mm)
{
 int ind, len;
 int kk;
 double *row, *col;
 double dot = 0.;

 kk = MAX(aa.lownode(ii), aa.lownode(jj));
 len = mm - kk;
 if (len > 0){
    row = &(lu.l(ii,kk));
    col = &(lu.u(kk,jj));
    /* for (i = kk;   i < mm;   i++) */
    for (ind = 0;   ind < len;   ind++)
        dot += row[-ind] * col[ind];
 }
 return dot;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
